"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const promisify_1 = __importDefault(require("../lib/promisify"));
const fs_1 = __importDefault(require("fs"));
const webpack_1 = __importDefault(require("webpack"));
const nanoid_1 = __importDefault(require("nanoid"));
const next_config_1 = __importDefault(require("next-server/next-config"));
const constants_1 = require("next-server/constants");
const webpack_2 = __importDefault(require("./webpack"));
const access = promisify_1.default(fs_1.default.access);
const writeFile = promisify_1.default(fs_1.default.writeFile);
async function generateBuildId(generate, fallback) {
    let buildId = await generate();
    // If there's no buildId defined we'll fall back
    if (buildId === null) {
        buildId = fallback();
    }
    if (typeof buildId !== 'string') {
        throw new Error('generateBuildId did not return a string. https://err.sh/zeit/next.js/generatebuildid-not-a-string');
    }
    return buildId.trim();
}
async function ensureProjectDirectoryIsWriteAble(dir) {
    try {
        await access(dir, (fs_1.default.constants || fs_1.default).W_OK);
    }
    catch (err) {
        throw new Error('Build directory is not writeable. https://err.sh/zeit/next.js/build-dir-not-writeable');
    }
}
async function build(dir, conf = null, lambdas = false) {
    const config = next_config_1.default(constants_1.PHASE_PRODUCTION_BUILD, dir, conf);
    const lambdasOption = config.lambdas ? config.lambdas : lambdas;
    const distDir = path_1.join(dir, config.distDir);
    const buildId = await generateBuildId(config.generateBuildId, nanoid_1.default);
    await ensureProjectDirectoryIsWriteAble(dir);
    try {
        const configs = await Promise.all([
            webpack_2.default(dir, { buildId, isServer: false, config, lambdas: lambdasOption }),
            webpack_2.default(dir, { buildId, isServer: true, config, lambdas: lambdasOption })
        ]);
        await runCompiler(configs);
        await writeBuildId(distDir, buildId);
    }
    catch (err) {
        console.error(`> Failed to build`);
        throw err;
    }
}
exports.default = build;
function runCompiler(compiler) {
    return new Promise(async (resolve, reject) => {
        const webpackCompiler = await webpack_1.default(await compiler);
        webpackCompiler.run((err, stats) => {
            if (err) {
                console.log(Object.assign({}, err));
                console.log(...stats.errors);
                return reject(err);
            }
            let buildFailed = false;
            for (const stat of stats.stats) {
                for (const error of stat.compilation.errors) {
                    buildFailed = true;
                    console.error('ERROR', error);
                    console.error('ORIGINAL ERROR', error.error);
                }
                for (const warning of stat.compilation.warnings) {
                    console.warn('WARNING', warning);
                }
            }
            if (buildFailed) {
                return reject(new Error('Webpack errors'));
            }
            resolve();
        });
    });
}
async function writeBuildId(distDir, buildId) {
    const buildIdPath = path_1.join(distDir, constants_1.BUILD_ID_FILE);
    await writeFile(buildIdPath, buildId, 'utf8');
}
