"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const del_1 = __importDefault(require("del"));
const recursive_copy_1 = __importDefault(require("recursive-copy"));
const mkdirp_then_1 = __importDefault(require("mkdirp-then"));
const path_1 = require("path");
const fs_1 = require("fs");
const next_config_1 = __importDefault(require("next-server/next-config"));
const constants_1 = require("next-server/constants");
const render_1 = require("next-server/dist/server/render");
const asset_1 = require("next-server/asset");
const envConfig = __importStar(require("next-server/config"));
async function default_1(dir, options, configuration) {
    function log(message) {
        if (options.silent)
            return;
        console.log(message);
    }
    dir = path_1.resolve(dir);
    const nextConfig = configuration || next_config_1.default(constants_1.PHASE_EXPORT, dir);
    const distDir = path_1.join(dir, nextConfig.distDir);
    log(`> using build directory: ${distDir}`);
    if (!fs_1.existsSync(distDir)) {
        throw new Error(`Build directory ${distDir} does not exist. Make sure you run "next build" before running "next start" or "next export".`);
    }
    const buildId = fs_1.readFileSync(path_1.join(distDir, constants_1.BUILD_ID_FILE), 'utf8');
    const pagesManifest = require(path_1.join(distDir, constants_1.SERVER_DIRECTORY, constants_1.PAGES_MANIFEST));
    const pages = Object.keys(pagesManifest);
    const defaultPathMap = {};
    for (const page of pages) {
        // _document and _app are not real pages.
        if (page === '/_document' || page === '/_app') {
            continue;
        }
        if (page === '/_error') {
            defaultPathMap['/404.html'] = { page };
            continue;
        }
        defaultPathMap[page] = { page };
    }
    // Initialize the output directory
    const outDir = options.outdir;
    await del_1.default(path_1.join(outDir, '*'));
    await mkdirp_then_1.default(path_1.join(outDir, '_next', buildId));
    // Copy static directory
    if (fs_1.existsSync(path_1.join(dir, 'static'))) {
        log('  copying "static" directory');
        await recursive_copy_1.default(path_1.join(dir, 'static'), path_1.join(outDir, 'static'), { expand: true });
    }
    // Copy .next/static directory
    if (fs_1.existsSync(path_1.join(distDir, constants_1.CLIENT_STATIC_FILES_PATH))) {
        log('  copying "static build" directory');
        await recursive_copy_1.default(path_1.join(distDir, constants_1.CLIENT_STATIC_FILES_PATH), path_1.join(outDir, '_next', constants_1.CLIENT_STATIC_FILES_PATH));
    }
    // Get the exportPathMap from the config file
    if (typeof nextConfig.exportPathMap !== 'function') {
        console.log(`> No "exportPathMap" found in "${constants_1.CONFIG_FILE}". Generating map from "./pages"`);
        nextConfig.exportPathMap = async (defaultMap) => {
            return defaultMap;
        };
    }
    // Start the rendering process
    const renderOpts = {
        dir,
        buildId,
        nextExport: true,
        assetPrefix: nextConfig.assetPrefix.replace(/\/$/, ''),
        distDir,
        dev: false,
        staticMarkup: false,
        hotReloader: null
    };
    const { serverRuntimeConfig, publicRuntimeConfig } = nextConfig;
    if (publicRuntimeConfig) {
        renderOpts.runtimeConfig = publicRuntimeConfig;
    }
    envConfig.setConfig({
        serverRuntimeConfig,
        publicRuntimeConfig
    });
    // set the assetPrefix to use for 'next/asset'
    asset_1.setAssetPrefix(renderOpts.assetPrefix);
    // We need this for server rendering the Link component.
    global.__NEXT_DATA__ = {
        nextExport: true
    };
    const exportPathMap = await nextConfig.exportPathMap(defaultPathMap, { dev: false, dir, outDir, distDir, buildId });
    const exportPaths = Object.keys(exportPathMap);
    for (const path of exportPaths) {
        log(`> exporting path: ${path}`);
        if (!path.startsWith('/')) {
            throw new Error(`path "${path}" doesn't start with a backslash`);
        }
        const { page, query = {} } = exportPathMap[path];
        const req = { url: path };
        const res = {};
        let htmlFilename = `${path}${path_1.sep}index.html`;
        if (path_1.extname(path) !== '') {
            // If the path has an extension, use that as the filename instead
            htmlFilename = path;
        }
        else if (path === '/') {
            // If the path is the root, just use index.html
            htmlFilename = 'index.html';
        }
        const baseDir = path_1.join(outDir, path_1.dirname(htmlFilename));
        const htmlFilepath = path_1.join(outDir, htmlFilename);
        await mkdirp_then_1.default(baseDir);
        const html = await render_1.renderToHTML(req, res, page, query, renderOpts);
        fs_1.writeFileSync(htmlFilepath, html, 'utf8');
    }
    // Add an empty line to the console for the better readability.
    log('');
}
exports.default = default_1;
